<?php declare(strict_types=1);

namespace Database;

use Config\Config;
use Exceptions\DatabaseConnectionException;
use PDO;

class Connection
{

    protected PDO $conn;
    private Config $configManager;

    /**
     * @throws DatabaseConnectionException
     */
    public function __construct()
    {
        $this->configManager = new Config();
        $this->connect();
    }

    private function configure(): string
    {
        $hostname = $this->configManager->getDbHost();
        $dbName = $this->configManager->getDbName();
        $port = $this->configManager->getDbPort();

        return "mysql:host=$hostname;port=$port;dbname=$dbName";

    }

    /**
     * @throws DatabaseConnectionException Thrown when the connection fails.
     */
    public function connect(): void
    {

        $username = $this->configManager->getDbUser();
        $password = $this->configManager->getDbPassword();

        try {
            $this->conn = new \PDO($this->configure(), $username, $password);
        } catch (\PDOException $e) {
            throw new DatabaseConnectionException("A connection to the underlying backend could not be established. Please try again later", $e->getCode(), $e->getPrevious());
        }

    }

    public function getConnection(): PDO
    {
        return $this->conn;
    }

}