<?php declare(strict_types=1);

namespace Utils;

class StructuredResponseBuilder
{

    /**
     * @var bool If true, the response payload is automatically formated as an error
     */
    private bool $isErrorResponse = false;

    /**
     * @var int Only used if isErrorResponse is True. Sets the HTTP response code for the error.
     */
    private int $isErrorResponseCode;

    /**
     * @var string Only used if isErrorResponse is True. Sets the HTTP error message for the error.
     */
    private string $isErrorMessage;

    /**
     * @var string Sets the optional message to be sent within the payload.
     */
    private string $optionalMessage;

    /**
     * @var array Sets the payload to be sent within the response.
     */
    private array $payload = [];


    /**
     * @return bool
     */
    public function isError(): bool
    {
        return $this->isErrorResponse;
    }

    /**
     * @param bool $isErrorResponse
     * @return $this
     */
    public function setError(bool $isErrorResponse = true): self
    {
        $this->isErrorResponse = $isErrorResponse;
        return $this;
    }

    /**
     * @return int
     */
    public function getErrorCode(): int
    {
        return $this->isErrorResponseCode;
    }

    /**
     * @param int $isErrorResponseCode
     * @return $this
     */
    public function setErrorCode(int $isErrorResponseCode): self
    {
        $this->isErrorResponseCode = $isErrorResponseCode;
        return $this;
    }

    /**
     * @return string
     */
    public function getErrorMessage(): string
    {
        return $this->isErrorMessage;
    }

    /**
     * @param string $isErrorMessage
     * @return $this
     */
    public function setErrorMessage(string $isErrorMessage): self
    {
        $this->isErrorMessage = $isErrorMessage;
        return $this;
    }

    /**
     * @return string
     */
    public function getOptionalMessage(): string
    {
        return $this->optionalMessage;
    }

    /**
     * @param string $optionalMessage
     * @return $this
     */
    public function setOptionalMessage(string $optionalMessage): self
    {
        $this->optionalMessage = $optionalMessage;
        return $this;
    }

    /**
     * @return array
     */
    public function getPayload(): array
    {
        return $this->payload;
    }

    /**
     * @param array $payload
     * @return $this
     */
    public function setPayload(array $payload): self
    {
        $this->payload = $payload;
        return $this;
    }


    public function build(): string
    {
        return json_encode([
            "error" => $this->isError(),
            "code" => (!$this->isError()) ? 'NaN' : $this->getErrorCode(),
            "message" => ($this->isError()) ? $this->getErrorMessage() : $this->getOptionalMessage(),
            "payload" => $this->getPayload()
        ]);
    }
}